
/* ------------------------ */
/* regular expressions */
/* ------------------------ */

numeric			((\d+(\.\d*)?)|(\.\d+))
alphanum_u		[A-Za-z_0-9]
word 			{alphanum_u}+
name 			[A-Za-z_]{alphanum_u}*

single_quoted_string 	['](?:[^'\\]|\\.)*[']
double_quoted_string 	["](?:[^"\\]|\\.)*["]

%x jm jm_nest jm_line jm_template

%options flex

%%
/* ------------------------ */
/* lexical vocabulary */
/* ------------------------ */

<jm,jm_nest,jm_line>{single_quoted_string} 		return 'JM';
<jm,jm_nest,jm_line>{double_quoted_string} 		return 'JM';
<jm,jm_nest,jm_line>"`" 						this.begin('jm_template'); return 'JM';
<jm,jm_nest,jm_line>"{"							this.begin('jm_nest'); return 'JM';

<jm_nest>"}"				this.popState(); return 'JM';

<jm_template>"${"			this.begin('JM'); return 'JM';
<jm_template>"}"			this.popState(); return 'JM';

<jm>"}"							this.popState(); return '}';

<jm_line>\n 					this.popState();

<jm,jm_nest,jm_line>.		return 'JM';


/* escape '@' character */
[@][@]					return 'VERBATIM';
[@]\s*[/][/][^\n]*\n 	{ /* ignore line comments */ }
[@][#][^\n]*\n 	{ /* ignore line comments */ }
/*[@]\s*[/][*][^]*[*][/] 	{ /* ignore block comments *       / }  /**/

"@{"				this.begin('jm');   return '@{';
"@.{"				this.begin('jm');   return '@.{';
[@][.][ \t]+		this.begin('jm_line'); return '@.';
"@let"				this.begin('jm_line'); return '@let'
"@def"	 			this.begin('jm_line'); return '@def';
"@include"			this.begin('jm_line'); return '@include';
"@if"				this.begin('jm_line'); return '@if';
"@elseif"			this.begin('jm_line'); return '@elseif';
"@else"				return '@else';
"@end"\n			return '@end';


<INITIAL>[^@]+				return 'VERBATIM';
(?:.|\n) 					return 'VERBATIM';


<<EOF>> 		return 'EOF'
